'use strict';function _asyncToGenerator(fn) {return function () {var gen = fn.apply(this, arguments);return new Promise(function (resolve, reject) {function step(key, arg) {try {var info = gen[key](arg);var value = info.value;} catch (error) {reject(error);return;}if (info.done) {resolve(value);} else {return Promise.resolve(value).then(function (value) {step("next", value);}, function (err) {step("throw", err);});}}return step("next");});};}
const sRavelInstance = Symbol.for('_ravelInstance');

const defaultOptions = {
  expire: null };


/**
                     * Koa Middleware which caches the body, lastModified, etag, etc.
                     * of a request in redis for a specified amount of time (or indefinitely).
                     * If cached, the response will be retrieved from the cache rather than
                     * executing the next element in the middleware chain.
                     * Borrowed heavily from https://github.com/coderhaoxin/koa-redis-cache
                     *
                     * @private
                     */let
ResponseCacheMiddleware = class ResponseCacheMiddleware {
  /**
                                                          * Creates an instance of Cache.
                                                          *
                                                          * @param {Ravel} ravelInstance - An instance of a Ravel app.
                                                          * @private
                                                          */
  constructor(ravelInstance) {
    this[sRavelInstance] = ravelInstance;
  }

  /**
     * Wraps setex in a Promise.
     *
     * @param {string} key - The key to set.
     * @param {number | undefined} expiry - The expiry time (optional).
     * @param {string | Buffer} body - The value.
     * @private
     */
  setex(key, expiry, body) {
    return new Promise((resolve, reject) => {
      const cb = (err, result) => {
        if (err) {
          this[sRavelInstance].log.error('Could not cache response.', err.stack);
          return reject(new Error('Could not cache response'));
        }
        resolve(result);
      };
      typeof expiry === 'number' ?
      this[sRavelInstance].kvstore.setex(key, expiry, body, cb) : this[sRavelInstance].kvstore.set(key, body, cb);
    });
  }

  /**
     * Wraps get in a Promise.
     *
     * @param {string} key - The key to get.
     * @private
     */
  get(key) {
    return new Promise((resolve, reject) => {
      this[sRavelInstance].kvstore.get(key, (err, result) => {
        if (err || result === null) {
          this[sRavelInstance].log.trace(`Could not get cached response for key ${key}`);
          return reject(new Error(`Could not get cached response for key ${key}`));
        }
        resolve(result);
      });
    });
  }

  /**
     * Populates a response from the cache, if possible, and resolves.
     * Rejects if there is no cached response, or if something goes wrong.
     *
     * @param {Context} ctx - A koa context.
     * @param {string} key - The key which should contain the response body cache.
     * @param {string} metaKey - The key which should contain the response metadata cache.
     * @returns {Promise} Resolves if the response was successfully populated. Rejects otherwise.
     * @private
     */
  getFromCache(ctx, key, metaKey) {var _this = this;return _asyncToGenerator(function* () {
      const value = yield _this.get(key); // this will throw if the value is missing
      ctx.response.status = 200;
      const metadata = JSON.parse((yield _this.get(metaKey)));
      ctx.response.set('X-Ravel-Cache', 'true');
      ctx.response.type = metadata.type;
      if (metadata.lastModified) ctx.response.lastModified = metadata.lastModified;
      ctx.response.etag = metadata.etag;
      ctx.response.body = value;})();
  }

  /**
     * Caches a response body and metadata in Redis.
     *
     * @param {Context} ctx - A koa context.
     * @param {string} key - The key which should contain the response body cache.
     * @param {string} metaKey - The key which should contain the response metadata cache.
     * @param {Object} options - Options, such as the cache expiry.
     * @returns {Promise} Resolves if the response was successfully populated. Rejects otherwise.
     * @private
     */
  cacheBody(ctx, key, metaKey, options) {var _this2 = this;return _asyncToGenerator(function* () {
      let body = ctx.response.body;
      if (ctx.request.method !== 'GET' || ctx.response.status !== 200 || !body) {
        return;
      }
      if (typeof body === 'string') {
        // string
        if (Buffer.byteLength(body) > options.maxLength) return;
        yield _this2.setex(key, options.expire, body);
      } else if (Buffer.isBuffer(body)) {
        // buffer
        if (body.length > options.maxLength) return;
        yield _this2.setex(key, options.expire, body);
      } else if (typeof body === 'object' && ctx.response.type === 'application/json') {
        // json
        body = JSON.stringify(body);
        if (Buffer.byteLength(body) > options.maxLength) return;
        yield _this2.setex(key, options.expire, body);
      } else if (typeof body.pipe === 'function') {
        _this2[sRavelInstance].log.warn('@cache does not currently support streamed bodies.');
        return;
      } else {
        return;
      }
      yield _this2.setex(metaKey, options.expire, JSON.stringify({
        type: ctx.response.type,
        lastModified: ctx.response.lastModified || null,
        etag: ctx.response.get('Etag') }));})();

  }

  /**
     * Configurablel middleware for caching responses and re-serving them.
     *
     * @param {Object} inOptions - Options for configuring the caching middleware.
     * @returns {AsyncFunction} The caching middleware.
     * @private
     */
  middleware(inOptions = {}) {var _this3 = this;
    let options = Object.create(null);
    options = Object.assign(options, defaultOptions);
    options = Object.assign(options, inOptions);
    return (() => {var _ref = _asyncToGenerator(function* (ctx, next) {
        const key = `ravel-cache-${ctx.request.method}-${ctx.request.url}`;
        const metaKey = key + ':type';
        try {
          yield _this3.getFromCache(ctx, key, metaKey);
        } catch (e) {
          yield next();
          try {
            yield _this3.cacheBody(ctx, key, metaKey, options);
          } catch (e2) {
            _this3[sRavelInstance].log.error(e2);
          }
        }
      });return function (_x, _x2) {return _ref.apply(this, arguments);};})();
  }};


module.exports = ResponseCacheMiddleware;